/**
 * CeboMsr - C++ example - Streaming Input.
 *
 * Multiple inputs are read multiple times at 300 Hz
 * and printed to the console.
 *
 * (c) 2013 Cesys GmbH
 * @author Thomas Hoppe
 */

#include <iostream>
#include <stdexcept>
#include <cmath>

#include <cebomsrpp.h>

using namespace std;
using namespace CeboMsr;

/**
 * Read blocks of data from device.
 */
void streamingInputTestLC(Device &device) {
	// Construct selection that contains inputs to read from.
	InputVector inputs;
	inputs
		<< device.getSingleEndedInputs().at(0)
		<< device.getSingleEndedInputs().at(1)
		<< device.getDifferentialInputs().at(1)
		<< device.getDigitalPorts().at(0)
		<< device.getDigitalPorts().at(1);

	// Prepare device with this collection ...
	device.setupInputFrame(inputs);

	// Start sampling ...
	device.startBufferedDataAcquisition(300, 20 * 25, false);

	// Read 20 x 25 frames using blocked read,
	// this function returns after *all* (25) requested frames are collected.
	for (int i = 0; i < 20; ++i) {
		// Read 25 frames ...
		InputFrameVector frames = device.readBlocking(25);

		// Write out the 1st one.
		const InputFrame &inFrame = frames[0];
		cout
			<< "DigitalPort #0: " << inFrame.getDigitalPort(0) << ", "
			<< "DigitalPort #1: " << inFrame.getDigitalPort(1) << ", "
			<< "SingleEnded #0: " << inFrame.getSingleEnded(0) << " V, "
			<< "SingleEnded #1: " << inFrame.getSingleEnded(1) << " V, "
			<< "Differential #1: " << inFrame.getDifferential(1) << " V"
			<< endl;
	}

	// Stop the DAQ.
	device.stopDataAcquisition();
}

/**
 * Read blocks of data from device.
 */
void streamingInputTestStick(Device &device) {
	// Construct selection that contains inputs to read from.
	InputVector inputs;
	inputs
		<< device.getSingleEndedInputs().at(0)
		<< device.getSingleEndedInputs().at(1)
		<< device.getDigitalPorts().at(0);

	// Prepare device with this collection ...
	device.setupInputFrame(inputs);

	// Start sampling ...
	device.startBufferedDataAcquisition(300, 20 * 25, false);

	// Read 20 x 25 frames using blocked read,
	// this function returns after *all* (25) requested frames are collected.
	for (int i = 0; i < 20; ++i) {
		// Read 25 frames ...
		InputFrameVector frames = device.readBlocking(25);

		// Write out the 1st one.
		const InputFrame &inFrame = frames[0];
		cout
			<< "DigitalPort #0: " << inFrame.getDigitalPort(0) << ", "
			<< "SingleEnded #0: " << inFrame.getSingleEnded(0) << " V, "
			<< "SingleEnded #1: " << inFrame.getSingleEnded(1) << " V"
			<< endl;
	}

	// Stop the DAQ.
	device.stopDataAcquisition();
}

/**
 * Application entry point.
 */
int main() {
	// Nearly all methods can raise exceptions. For this example, the error handling is done at this single point.
	try {
		// Search for devices ...
		DeviceVector devices = LibraryInterface::enumerate(DeviceType::All);

		// If at least one has been found, use the first one ...
		if (!devices.empty()) {
			Device device = devices[0];

			// Open device, nothing can be done without doing this.
			device.open();

			// Do the streaming ...
			if (device.getDeviceType() == DeviceType::CeboLC)
				streamingInputTestLC(device);
			else if (device.getDeviceType() == DeviceType::CeboStick)
				streamingInputTestStick(device);
			else
				throw std::runtime_error("No supported device detected.");

			// Finalize device usage, this free's up the device, so it can be used
			// again, including other applications.
			device.close();
		}
	} // These exceptions are currently used to report different problems ...
	catch (const std::invalid_argument &e) { cout << "Invalid argument exception raised: " << e.what() << endl; }
	catch (const std::out_of_range &e) { cout << "Out of range exception raised: " << e.what() << endl; }
	catch (const std::logic_error &e) { cout << "Logic error exception raised: " << e.what() << endl; }
	catch (const std::runtime_error &e) { cout << "Runtime/IO error exception raised: " << e.what() << endl; }
	catch (const std::exception &e) { cout << "Unknown exception raised: " << e.what() << endl; }

	return 0;
}
