#!/usr/bin/env python
from CeboMsrApiPython import LibraryInterface, DeviceType

# CeboMsr - Python example - Info.
#
# Print out various information about the device.
#
# @author Thomas Hoppe

def printInformation(device):
    """
    Print various information to the console.
    device: Device to use for test.
    """
    # Put out some device specific information.
    print("Device type: %s" % device.getDeviceType().getName())
    print("USB base ver: %s" % LibraryInterface.getUsbBaseVersion())
    print("API vers: %s" % LibraryInterface.getApiVersion())
    print("Firmware ver: %s" % device.getFirmwareVersion())
    print("Identifier: %s" % device.getIdentifier())
    print("Serial number: %s" % device.getSerialNumber())
    print("Temperature: %.2f" % device.getTemperature())

    # Get the real current of the reference current sources.
    for source in device.getCurrentSources():
        print("Reference current of %s: %.2f uA" % (
            source.getName(),
            source.getReferenceCurrent()))

    # Retrieve information about single ended input #0.
    # This works on all analog inputs and outputs.
    se0 = device.getSingleEndedInputs()[0]
    print("Info for single ended input %s:" % se0.getName())
    print("  Min. ICD: %d us" % se0.getMinInterChannelDelay())
    print("  Current range: %.2f V to %.2f V" % (
        se0.getRange().getMinValue(),
        se0.getRange().getMaxValue()))
    print("  Supported ranges:")
    for i in range(len(se0.getSupportedRanges())):
        rng = se0.getSupportedRanges()[i]
        print("    Range #%d range: %.2f V to %.2f V, Def. ICD: %d us" % (
            i,
            rng.getMinValue(),
            rng.getMaxValue(),
            se0.getDefaultInterChannelDelay(rng)))

    # Get info for digital port #0.
    dp0 = device.getDigitalPorts()[0]
    print("Info for digital port %s:" % dp0.getName())
    print("  Count of I/O's: %d" % dp0.getIoCount())
    for i in range(dp0.getIoCount()):
        print("    I/O #%d: %s" % (i, dp0.getIoName(i)))

def main():
    """
    The examples main method.
    """
    try:
        # Search for devices ...
        devices = LibraryInterface.enumerate(DeviceType.All)

        # If at least one has been found, use the first one ...
        if (len(devices) > 0):
            device = devices[0]

            # Open device, nothing can be done without doing this.
            device.open()

            # Put out info.
            printInformation(device)

            # Finalize device usage, this free's up the device, so it can be used
            # again, including other applications.
            device.close()
    # These exceptions are currently used to report different problems ...
    except AttributeError as e:
        print(e)
    except IndexError as e:
        print(e)
    except SystemError as e:
        print(e)
    except IOError as e:
        print(e)

if (__name__ == "__main__"):
    main()