package com.cesys.cebo.cebomsr;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * Offers functionality for analog outputs. Direct construction
 * is not possible. Instances can be retrieved calling getAnalogOutputs()
 * of the respective {@link Device} class.
 *
 * @author Thomas Hoppe
 */
public final class AnalogOutput implements Output {
	private final List<Range> _supportedRanges = new ArrayList<Range>();
	private final Device _device;
	private final int _interfaceId;
	private final int _index;
	private final String _name;

	int getInterfaceId() {
		return _interfaceId;
	}

	int getIndex() {
		return _index;
	}

	AnalogOutput(Device device, int interfaceId, int index) throws IOException {
		_device = device;
		_interfaceId = interfaceId;
		_index = index;

		int[] rangeId = new int[1];
		for (int i = 0;; ++i) {
			SystemConnector.checkError(SystemConnector.CeboMsrEnumerateRanges(_device.getHandle(), getInterfaceId(), i, rangeId));
			if (rangeId[0] == SystemConnector.CeboMsrRangeInvalid)
				break;

			float[] minValue = new float[1], maxValue = new float[1];
			SystemConnector.checkError(SystemConnector.CeboMsrGetRangeMinMax(device.getHandle(), getInterfaceId(), rangeId[0], minValue, maxValue));
			_supportedRanges.add(new Range(rangeId[0], minValue[0], maxValue[0], 0));
		}
		byte[] tmp = new byte[SystemConnector.MaxIdentLength];
		SystemConnector.checkError(SystemConnector.CeboMsrGetPeripheralName(device.getHandle(), interfaceId, tmp));
		_name = Utils.convertByteArrayToString(tmp);
	}

	/**
	 * Return list of the supported ranges.
	 */
	public List<Range> getSupportedRanges() {
		return Collections.unmodifiableList(_supportedRanges);
	}

	/**
	 * Set new range.
	 */
	public void setParameters(Range range) throws IOException {
		// No device currently supports this, included for future functionality.
//		int r[] = new int[1];
//		SystemConnector.checkError(SystemConnector.CeboMsrSetAnalogInputParameters(_device.getHandle(), getInterfaceId(), range.getInterfaceId(), -1, r));
	}

	/**
	 * Read active range level.
	 */
	public Range getRange() throws IOException {
		// No device currently supports this, included for future functionality.
		return _supportedRanges.get(0);
	}

	/**
	 * Set voltage level directly.
	 * @param value Value in volt to set.
	 */
	public void write(float value) throws IOException {
		SystemConnector.checkError(SystemConnector.CeboMsrWriteAnalogOutput(_device.getHandle(), getInterfaceId(), value));
	}

	/**
	 * Returns name of the component.
	 */
	public String getName() {
		return _name;
	}

	public String toString() {
		return "AnalogOutput" + _index;
	}
}
